"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const lodash_1 = tslib_1.__importDefault(require("lodash"));
const unexpected_1 = tslib_1.__importDefault(require("unexpected"));
const src_1 = require("../src");
class WriteEightObserversPartialUpdate extends src_1.Benchmark {
    constructor(client, example) {
        super(client, example);
        this.observers = [];
        this.updatedResponses = [];
        // This is a bit hacky, but does a decent job of exploring partial updates.
        const update = updateScalarFields(example.partials[7].response);
        for (let i = 0; i < 8; i++) {
            this.updatedResponses.push(lodash_1.default.merge({}, example.partials[i].response, update));
        }
    }
    async setup() {
        await this.client.write(this.example);
        for (let i = 0; i < 8; i++) {
            this.observers.push(this.client.observe(this.example.partials[i]));
        }
    }
    async run() {
        await this.client.write(Object.assign({}, this.example.partials[7], { response: this.updatedResponses[7] }));
    }
    async verify() {
        for (let i = 0; i < 8; i++) {
            const observer = this.observers[i];
            const updatedResponse = this.updatedResponses[i];
            unexpected_1.default(observer.mostRecentResult().data, 'to satisfy', updatedResponse);
        }
    }
    async teardown() {
        for (const observer of this.observers) {
            observer.unsubscribe();
        }
    }
}
WriteEightObserversPartialUpdate.metadata = {
    title: `write (8 observers, partial update)`,
};
exports.default = WriteEightObserversPartialUpdate;
/**
 * Swaps out any scalar fields (except id and __typename), replacing them with
 * new values, in order to simulate an updated response.
 */
function updateScalarFields(response) {
    return lodash_1.default.cloneDeepWith(response, (value, key) => {
        if (key === 'id')
            return;
        if (key === '__typename')
            return;
        if (typeof value === 'string')
            return `new ${value}`;
        if (typeof value === 'number')
            return value + 1;
        if (typeof value === 'boolean')
            return !value;
    });
}
