"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const graphql_tag_1 = tslib_1.__importDefault(require("graphql-tag"));
const apollo_client_1 = require("apollo-client");
const apollo_link_1 = require("apollo-link");
const apollo_cache_inmemory_1 = require("apollo-cache-inmemory");
const src_1 = require("../src");
class ApolloObserver {
    constructor(observable) {
        this._mostRecentResult = null;
        this._subscription = observable.subscribe(({ data }) => {
            this._mostRecentResult = { data };
        }, error => {
            this._mostRecentResult = { errors: [error] };
        });
    }
    unsubscribe() {
        this._subscription.unsubscribe();
    }
    mostRecentResult() {
        return this._mostRecentResult;
    }
}
class ApolloInMemory extends src_1.Client {
    constructor(cache = new apollo_cache_inmemory_1.InMemoryCache()) {
        super();
        this.apollo = new apollo_client_1.ApolloClient({
            cache,
            link: new apollo_link_1.ApolloLink(),
        });
    }
    transformRawExample(rawExample) {
        return {
            operation: graphql_tag_1.default(rawExample.operation),
            response: rawExample.response,
            variables: rawExample.variables,
        };
    }
    async read({ operation, variables }) {
        try {
            return {
                data: this.apollo.readQuery({ query: operation, variables }),
            };
        }
        catch (error) {
            // Apollo throws if data is missing
            return null;
        }
    }
    async write({ operation, response, variables }) {
        this.apollo.writeQuery({
            data: response,
            query: operation,
            variables,
        });
    }
    observe({ operation, variables }) {
        const observable = this.apollo.watchQuery({
            query: operation,
            variables,
            fetchPolicy: 'cache-only',
        });
        return new ApolloObserver(observable);
    }
}
ApolloInMemory.metadata = {
    name: `Apollo (InMemory)`,
};
exports.ApolloInMemory = ApolloInMemory;
