"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const graphql = tslib_1.__importStar(require("graphql"));
const graphqlCompiler = tslib_1.__importStar(require("graphql-compiler"));
const relayCompiler = tslib_1.__importStar(require("relay-compiler"));
const relayRuntime = tslib_1.__importStar(require("relay-runtime"));
const src_1 = require("../src");
// https://github.com/facebook/relay/blob/v1.6.0/packages/relay-compiler/bin/RelayCompilerBin.js#L208-L214
const TRANSFORMS = {
    commonTransforms: relayCompiler.IRTransforms.commonTransforms,
    codegenTransforms: relayCompiler.IRTransforms.codegenTransforms,
    fragmentTransforms: relayCompiler.IRTransforms.fragmentTransforms,
    printTransforms: relayCompiler.IRTransforms.printTransforms,
    queryTransforms: relayCompiler.IRTransforms.queryTransforms,
};
class RelayObserver {
    constructor(client, operation) {
        this._mostRecentResult = null;
        this._onNewSnapshot = (snapshot) => {
            this._mostRecentResult = {
                data: snapshot.data,
            };
        };
        const initialSnapshot = client.lookup(operation.root);
        this._onNewSnapshot(initialSnapshot);
        this._disposable = client.subscribe(initialSnapshot, this._onNewSnapshot);
    }
    unsubscribe() {
        return this._disposable.dispose();
    }
    mostRecentResult() {
        return this._mostRecentResult;
    }
}
class Relay extends src_1.Client {
    constructor() {
        super(...arguments);
        this._client = new relayRuntime.Environment({
            network: relayRuntime.Network.create(async () => {
                throw new Error(`end-to-end queries are not supported`);
            }),
            store: new relayRuntime.Store(new relayRuntime.RecordSource()),
        });
    }
    transformRawExample(rawExample, schema) {
        const schemaAst = graphql.buildASTSchema(graphql.parse(schema));
        let context = new graphqlCompiler.CompilerContext(schemaAst);
        // In some queries, we leverage Hermes' @static directive; but Relay doesn't
        // like unknown directives. Quick hack to remove it.
        const scrubbedOperation = rawExample.operation.replace(/@static/g, '');
        context = context.addAll(graphqlCompiler.Parser.parse(schemaAst, scrubbedOperation));
        const artifacts = relayCompiler.compileRelayArtifacts(context, TRANSFORMS);
        const requests = artifacts.filter(a => a.kind !== 'Fragment');
        if (requests.length !== 1) {
            throw new Error(`Expected the query document to contain only one operation`);
        }
        const request = relayRuntime.getRequest(requests[0]);
        const operation = relayRuntime.createOperationSelector(request, rawExample.variables);
        return {
            operation,
            response: rawExample.response,
        };
    }
    async read({ operation }) {
        const { data } = this._client.lookup(operation.root);
        return data ? { data } : null;
    }
    async write({ operation, response }) {
        this._client.commitPayload(operation, response);
    }
    observe({ operation }) {
        return new RelayObserver(this._client, operation);
    }
}
Relay.metadata = {
    name: `Relay`,
};
exports.Relay = Relay;
