"use strict";
const tslib_1 = require("tslib");
const path = tslib_1.__importStar(require("path"));
const src_1 = require("../src");
// Collect all files for examples in this directory (recursively)
const exampleContext = require.context('.', true, /(metadata\.json|operation\.gql|response\.json|schema\.gql)$/);
const examplesByDirname = {};
// Walk all asset files, and group them into examples (by dirname).
exampleContext.keys().forEach(assetPath => {
    const examplePath = path.dirname(assetPath);
    if (!examplesByDirname[examplePath]) {
        examplesByDirname[examplePath] = {};
    }
    const assetType = path.basename(assetPath, path.extname(assetPath));
    const assetContent = exampleContext(assetPath);
    if (assetType == 'metadata') {
        examplesByDirname[examplePath] = Object.assign({}, examplesByDirname[examplePath], assetContent);
    }
    else {
        examplesByDirname[examplePath][assetType] = assetContent;
    }
});
const examples = [];
for (const dirname of Object.keys(examplesByDirname)) {
    const example = examplesByDirname[dirname];
    if (!isRawExample(example)) {
        throw new Error(`examples/${dirname} is not a fully formed example. Required: title, operation, response, schema`);
    }
    example.partials = src_1.generatePartialExamples(example);
    examples.push(example);
}
function isRawExample(value) {
    return value.title && value.operation && value.response && value.schema;
}
module.exports = examples;
