"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ink_1 = require("ink");
const Table_1 = require("./Table");
class SuiteSummary extends ink_1.Component {
    render() {
        const { title } = this.props;
        return (React.createElement("div", null,
            React.createElement(ink_1.Indent, { size: 1 },
                React.createElement("div", null),
                React.createElement("div", null,
                    React.createElement(ink_1.Color, { yellow: true }, "GraphQL Client Benchmark Suite")),
                React.createElement("div", null),
                React.createElement("div", null,
                    "Example Query: ",
                    React.createElement(ink_1.Color, { blue: true }, title))),
            React.createElement("div", null),
            React.createElement(Table_1.Table, null,
                this._renderHeader(),
                this._renderRows())));
    }
    _renderHeader() {
        const { clients } = this.props;
        return (React.createElement(Table_1.Table.Row, null,
            React.createElement(Table_1.Table.Cell, { width: 40, height: 3, stretch: false, hAlign: "center" },
                React.createElement(ink_1.Color, null, " Benchmarks ")),
            clients.map(client => (React.createElement(Table_1.Table.Cell, { hAlign: "center" },
                React.createElement(ink_1.Color, { yellow: true },
                    " ",
                    client.name,
                    " "))))));
    }
    _renderRows() {
        const { clients, benchmarks } = this.props;
        return benchmarks.map(benchmark => (React.createElement(Table_1.Table.Row, null,
            React.createElement(Table_1.Table.Cell, { width: 40, height: 3, stretch: false },
                React.createElement(ink_1.Color, { blue: true },
                    " ",
                    benchmark.title,
                    " ")),
            clients.map(client => this._renderStats(benchmark.title, client.name)))));
    }
    _renderStats(benchmarkTitle, clientName) {
        const { results } = this.props;
        const result = results[benchmarkTitle] && results[benchmarkTitle][clientName];
        if (result && result.failure) {
            return (React.createElement(Table_1.Table.Cell, { hAlign: "center", vAlign: "middle" },
                React.createElement(ink_1.Color, { white: true, bgRed: true },
                    ' ',
                    "Error during ",
                    result.phase,
                    ' '),
                '\u200b'));
        }
        else if (!result || !result.stats) {
            const status = result ? result.phase : 'pending';
            return (React.createElement(Table_1.Table.Cell, { hAlign: "center", vAlign: "middle" },
                React.createElement(ink_1.Color, { dim: true },
                    status,
                    "\u2026")));
        }
        const { complete, stats } = result;
        return (React.createElement(Table_1.Table.Cell, null,
            React.createElement(ink_1.Color, { dim: !complete },
                '\u200b',
                "mean: ",
                stats.mean.toFixed(3),
                "ms \u00B1",
                stats.marginOfError.toFixed(3),
                "ms",
                '\n',
                '\u200b',
                "confidence: ",
                (100 - stats.percentRelativeMarginOfError).toFixed(2),
                "%",
                '\n',
                '\u200b',
                "count: ",
                stats.iterations,
                " range: ",
                stats.min.toFixed(0),
                "-",
                stats.max.toFixed(0),
                "ms")));
    }
}
exports.SuiteSummary = SuiteSummary;
