"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ink_1 = require("ink");
const benchmarks_1 = tslib_1.__importDefault(require("../../../benchmarks"));
const examples_1 = tslib_1.__importDefault(require("../../../examples"));
const clients_1 = tslib_1.__importDefault(require("../../../clients"));
const Root_1 = require("./Root");
const execution_1 = require("../../execution");
const [example] = examples_1.default;
async function main() {
    let componentEventHandler;
    function registerEventHandler(eventHandler) {
        componentEventHandler = eventHandler;
    }
    function reporter(event) {
        if (!componentEventHandler)
            return;
        componentEventHandler(event);
    }
    let suitePromise; // eslint-disable-line prefer-const
    function onExit() {
        if (!suitePromise)
            return;
        suitePromise.cancel();
    }
    const stopRendering = ink_1.render(React.createElement(Root_1.Root, { registerEventHandler: registerEventHandler, onExit: onExit }));
    suitePromise = execution_1.runSuite(reporter, benchmarks_1.default, clients_1.default, example);
    const canceled = await suitePromise;
    // Give it a chance to render a final time.
    await new Promise(resolve => setTimeout(resolve, 0));
    if (!canceled) {
        stopRendering();
    }
}
exports.main = main;
