"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const dapper = tslib_1.__importStar(require("@convoy/dapper"));
const chroma_js_1 = tslib_1.__importDefault(require("chroma-js"));
const BACKGROUND_COLOR_SCALE = chroma_js_1.default.scale([
    chroma_js_1.default('#4aa54f').luminance(0.75),
    chroma_js_1.default('#f8cc1c').luminance(0.75),
    chroma_js_1.default('#ee251a').luminance(0.75),
]);
const MODES = {
    pending: ({ props }) => !props.phase && !props.stats,
    active: ({ props }) => !!props.phase && !props.complete,
    iterating: ({ props }) => !!props.stats && !props.complete,
    complete: ({ props }) => props.complete,
    error: ({ props }) => !!props.failure,
};
const STYLES = dapper.compile({
    root: {
        transition: 'all 250ms ease-in-out',
        willChange: 'transition',
        $pending: {
            color: '#5b666b',
        },
        $active: {
            color: '#f7f9fa',
        },
        $iterating: {
            backgroundColor: '#14a9e6',
        },
        $complete: {
            backgroundColor: '#f7f9fa',
            color: '#0b2127',
        },
        $error: {
            backgroundColor: '#a11a12',
            color: '#f7f9fa',
        },
    },
    status: {
        textAlign: 'center',
    },
    error: {
        textAlign: 'center',
    },
});
class ResultsCell extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.styles = dapper.reactTo(this, STYLES, MODES);
    }
    render() {
        const { className, stats, failure } = this.props;
        let content;
        if (failure) {
            content = this._renderFailure();
        }
        else if (stats) {
            content = this._renderStats();
        }
        else {
            content = this._renderPhase();
        }
        return (React.createElement("td", { className: `${this.styles.root} ${className}`, style: this._inlineStyles() }, content));
    }
    _renderFailure() {
        const { phase } = this.props.failure;
        return React.createElement("div", { className: this.styles.error },
            "Error during ",
            phase);
    }
    _renderStats() {
        const { stats } = this.props;
        return (React.createElement(React.Fragment, null,
            React.createElement("div", null,
                "mean:",
                ' ',
                React.createElement("strong", null,
                    stats.mean.toFixed(3),
                    "ms \u00B1",
                    stats.marginOfError.toFixed(3),
                    "ms")),
            React.createElement("div", null,
                "confidence: ",
                (100 - stats.percentRelativeMarginOfError).toFixed(2),
                "%"),
            React.createElement("div", null,
                "count: ",
                stats.iterations,
                " range: ",
                stats.min.toFixed(0),
                "-",
                stats.max.toFixed(0),
                "ms")));
    }
    _renderPhase() {
        const { phase } = this.props;
        return React.createElement("div", { className: this.styles.status },
            phase || 'pending',
            "\u2026");
    }
    _inlineStyles() {
        const { percentOfRange } = this.props;
        if (typeof percentOfRange !== 'number')
            return undefined;
        // Color on a simple curve, assuming slower results are more likely to be
        // far away from the rest of the results.
        const colorOffset = Math.sqrt(percentOfRange);
        return {
            backgroundColor: BACKGROUND_COLOR_SCALE(colorOffset).css(),
        };
    }
}
exports.ResultsCell = ResultsCell;
