"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const dapper = tslib_1.__importStar(require("@convoy/dapper"));
const ResultsCell_1 = require("./ResultsCell");
const STYLES = dapper.compile({
    root: {
        borderCollapse: 'collapse',
        'td, th': {
            padding: `5px 10px`,
            border: '1px solid #0b2127',
        },
        th: {
            color: '#e6e8e9',
            fontWeight: 'normal',
        },
    },
    infoCell: {
        width: 200,
    },
    resultsCell: {
        width: 300,
        height: 66,
    },
});
class SuiteSummary extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.styles = dapper.reactTo(this, STYLES);
    }
    render() {
        const { className, title } = this.props;
        return (React.createElement("table", { className: `${this.styles.root} ${className}` },
            React.createElement("thead", null, this._renderHeader()),
            React.createElement("tbody", null, this._renderRows())));
    }
    _renderHeader() {
        const { clients } = this.props;
        if (!clients)
            return null;
        return (React.createElement("tr", null,
            React.createElement("th", null, "Benchmarks"),
            clients.map(client => React.createElement("th", { key: client.name }, client.name))));
    }
    _renderRows() {
        const { clients, benchmarks } = this.props;
        if (!benchmarks)
            return null;
        return benchmarks.map(benchmark => (React.createElement("tr", { key: benchmark.title },
            React.createElement("th", { className: this.styles.infoColumn }, benchmark.title),
            clients.map(client => this._renderResults(benchmark.title, client.name)))));
    }
    _renderResults(benchmarkTitle, clientName) {
        const { results } = this.props;
        const result = (results[benchmarkTitle] && results[benchmarkTitle][clientName]) || {};
        return React.createElement(ResultsCell_1.ResultsCell, Object.assign({}, result, { className: this.styles.resultsCell, key: clientName }));
    }
}
exports.SuiteSummary = SuiteSummary;
